from bs4 import BeautifulSoup
import re
from urllib.request import urlopen

from rotten_tomatoes_scraper.rt_scraper import CelebrityScraper, MovieScraper


class WikiParser:
    def __init__(self):
        pass

    @staticmethod
    def extract_actresses_names():
        url_actresses = 'https://en.wikipedia.org/wiki/Category:American_film_actresses'
        page_actresses = urlopen(url_actresses)
        soup = BeautifulSoup(page_actresses, "lxml")
        data = soup.find_all('div', class_='mw-category-group')
        actresses = data[8].text.split('\n')
        actresses = [re.sub(r'\s\([^)]*\)', '', actress) for actress in actresses[1:]]
        return actresses

    @staticmethod
    def extract_actors_names():
        url_actors = 'https://en.wikipedia.org/wiki/Category:American_male_film_actors'
        page_actors = urlopen(url_actors)
        soup = BeautifulSoup(page_actors, "lxml")
        data = soup.find_all('div', class_='mw-category-group')
        actors = data[1].text.split('\n')
        actors = [re.sub(r'\s\([^)]*\)', '', actor) for actor in actors[1:]]
        return actors


def test_celebrity_scraper_01():
    celebrity_scraper = CelebrityScraper(celebrity_name='jack nicholson')
    celebrity_scraper.extract_metadata(section='filmography')
    movie_titles = celebrity_scraper.metadata['movie_titles']
    assert 'The Departed' in movie_titles


def test_celebrity_scraper_02():
    celebrity_scraper = CelebrityScraper(celebrity_name='meryl streep')
    celebrity_scraper.extract_metadata(section='highest')
    movie_titles = celebrity_scraper.metadata['movie_titles']
    assert 'Manhattan' in movie_titles


def test_celebrity_scraper_03():
    wiki_parser = WikiParser()
    actresses = wiki_parser.extract_actresses_names()
    celebrity_scraper = CelebrityScraper(celebrity_name=actresses[2])
    celebrity_scraper.extract_metadata(section='highest')
    movie_titles = celebrity_scraper.metadata['movie_titles']
    assert 'Manhattan' in movie_titles


def test_movie_scraper_01():
    movie_scraper = MovieScraper(movie_title='Manhattan')
    movie_scraper.extract_metadata()
    movie_genres = movie_scraper.movie_genre
    assert 'Comedy' in movie_genres


def test_movie_scraper_02():
    movie_scraper = MovieScraper(movie_title='Manhattan')
    movie_scraper.extract_metadata()
    movie_genres = movie_scraper.movie_genre
    assert 'Kids&Family' in movie_genres


def test_movie_scraper_03():
    movie_url = 'https://www.rottentomatoes.com/m/manhattan'
    movie_scraper = MovieScraper(movie_url=movie_url)
    movie_scraper.extract_metadata()
    movie_genres = movie_scraper.movie_genre
    assert 'Kids&Family' not in movie_genres


def test_movie_scraper_04():
    movie_url = 'https://www.rottentomatoes.com/m/marriage_story_2019'
    movie_scraper = MovieScraper(movie_url=movie_url)
    movie_scraper.extract_metadata()
    print(movie_scraper.metadata)


def test_movie_scraper_05():
    movie_scraper = MovieScraper(movie_title='VICKY CRISTINA BARCELONA')
    movie_scraper.extract_metadata()
    print(movie_scraper.metadata)

test_movie_scraper_05()



from bs4 import BeautifulSoup
import re
import requests
from urllib.request import urlopen


class RTScraper:
    BASE_URL = "https://www.rottentomatoes.com/api/private/v2.0"
    SEARCH_URL = "{base_url}/search".format(base_url=BASE_URL)

    def __init__(self):
        self.metadata = dict()
        self.url = None

    def extract_url(self):
        pass

    def extract_metadata(self, **kwargs):
        pass

    def _extract_section(self, section):
        pass

    @staticmethod
    def search(term, limit=10):
        r = requests.get(url=RTScraper.SEARCH_URL, params={"q": term, "limit": limit})
        r.raise_for_status()
        return r.json()


class MovieScraper(RTScraper):
    def __init__(self, **kwargs):
        RTScraper.__init__(self)
        self.movie_genre = None
        if 'movie_title' in kwargs.keys():
            self.movie_title = kwargs['movie_title']
            self.extract_url()
        if 'movie_url' in kwargs.keys():
            self.url = kwargs['movie_url']

    def extract_url(self):
        search_result = self.search(term=self.movie_title)
        url_movie = 'https://www.rottentomatoes.com' + search_result['movies'][0]['url']
        if len(search_result['movies']) > 1:
            print('There are several movie records matching the search criteria.. The selected url is: {}'.format(url_movie))
        self.url = url_movie

    def extract_metadata(self, columns=('Rating', 'Genre', 'Box Office', 'Studio')):
        movie_metadata = dict()
        page_movie = urlopen(self.url)
        soup = BeautifulSoup(page_movie, "lxml")

        # Score
        score = soup.find_all('div', class_='mop-ratings-wrap__half')
        movie_metadata['Score_Rotten'] = score[0].text.strip().replace('\n', '').split(' ')[0]
        movie_metadata['Score_Audience'] = score[1].text.strip().replace('\n', '').split(' ')[0]

        # Movie Info
        movie_info_section = soup.find_all('div', class_='media-body')
        soup_movie_info = BeautifulSoup(str(movie_info_section[0]), "lxml")
        movie_info_length = len(soup_movie_info.find_all('li', class_='meta-row clearfix'))

        for i in range(movie_info_length):
            x = soup_movie_info.find_all('li', class_='meta-row clearfix')[i]
            soup = BeautifulSoup(str(x), "lxml")
            label = soup.find('div', class_='meta-label subtle').text.strip().replace(':', '')
            value = soup.find('div', class_='meta-value').text.strip()
            if label in columns:
                if label == 'Box Office':
                    value = int(value.replace('$', '').replace(',', ''))
                if label == 'Rating':
                    value = re.sub(r'\s\([^)]*\)', '', value)
                if label == 'Genre':
                    value = value.replace(' ', '').replace('\n', '').split(',')
                movie_metadata[label] = value

        self.metadata = movie_metadata
        self.movie_genre = self.extract_genre(self.metadata)

    @staticmethod
    def extract_genre(metadata):
        try:
            if 'Genre' in metadata:
                movie_genre = metadata['Genre']
            else:
                movie_genre = ['None']

        except IOError:
            movie_genre = ['None']

        return movie_genre


class CelebrityScraper(RTScraper):
    def __init__(self, **kwargs):
        RTScraper.__init__(self)
        if 'celebrity_name' in kwargs.keys():
            self.celebrity_name = kwargs['celebrity_name']
            self.extract_url()
        if 'celebrity_url' in kwargs.keys():
            self.url = kwargs['celebrity_url']

    def extract_url(self):
        search_result = self.search(term=self.celebrity_name)
        url_celebrity = 'https://www.rottentomatoes.com' + search_result['actors'][0]['url']
        self.url = url_celebrity

    def _extract_section(self, section):
        page_celebrity = urlopen(self.url)
        soup = BeautifulSoup(page_celebrity, "lxml")
        selected_section = []
        try:
            if section == 'highest':
                selected_section = soup.find_all('p', class_='celebrity-highest__info')
            elif section == 'filmography':
                selected_section = soup.find_all('tbody', class_='celebrity-filmography__tbody')[0]
        except IOError:
            print('The parsing process returns an error.')

        return selected_section

    def extract_metadata(self, section):
        selected_section = self._extract_section(section=section)
        movie_titles = []
        if section == 'highest':
            for i in range(len(selected_section)):
                movie_titles.append(selected_section[i].text.split('\n')[2].strip())
        elif section == 'filmography':
            soup_filmography = BeautifulSoup(str(selected_section), 'lxml')
            for h in soup_filmography.find_all('a'):
                try:
                    movie_titles.append(h.text.strip())
                except IOError:
                    pass
        self.metadata['movie_titles'] = movie_titles
